package com.usoft.sso.utils;

import com.alibaba.fastjson.JSON;
import com.uas.sso.SSOConfig;
import com.uas.sso.SSOHelper;
import com.uas.sso.SSOToken;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;

import javax.annotation.PostConstruct;
import javax.servlet.http.HttpServletRequest;

/**
 * Sso 控制层 基类
 *
 * @author: wangcanyi
 * @date: 2018-08-16 10:20
 **/
public abstract class BaseSsoController {
	@Autowired
	private SsoProperties ssoProperties;

	@PostConstruct
	public void init() {
		if (StringUtils.isBlank(ssoProperties.getApp())) {
			throw new RuntimeException("应用号不能为空，请在Properties文件配置sso.app");
		}
		if (StringUtils.isBlank(ssoProperties.getSecretKey())) {
			throw new RuntimeException("秘钥不能为空，请在Properties文件配置sso.secret-key");
		}
		//初始化sso-core包里的SSOConfig信息
		SSOConfig ssoConfig = SSOHelper.getSSOService().getConfig();
		ssoConfig.setAppName(ssoProperties.getApp());
		ssoConfig.setSecretkey(ssoProperties.getSecretKey());
		SSOHelper.getSSOService().getConfig().setCookieBrowser(ssoProperties.isCookieBrowser());
	}

	/**
	 * 获取登录态
	 *
	 * @param request
	 * @return
	 */
	protected UserLoginState getUserLoginState(HttpServletRequest request) {
		SSOToken ssoToken = SSOHelper.getToken(request);
		if (ssoToken == null) {
			return null;
		}
		if (StringUtils.isBlank(ssoToken.getData())) {
			return null;
		}
		return JSON.parseObject(ssoToken.getData(), UserLoginState.class);
	}
}
